/**
 * @fileOverview BusInクラスを定義します。
 */
(function(){
var NS=mbedJS;
var MI=MiMicJS;

/**
 * BusInクラスです。
 * <a href="https://mbed.org/handbook/BusIn">mbed::BusIn</a>と同等の機能を持ちます。
 * @name mbedJS.BusIn
 * @constructor
 * @param {mbedJS.Mcu} i_mcu
 * インスタンスをバインドするMCUオブジェクトです。
 * @param {[PinName...]} i_params
 * ピンIDの配列を指定します。要素数の最大値は16です。
 * @param {HashMap|Generator} i_event
 * 非同期イベントハンドラの連想配列、又はGeneratorです。
 * <p>
 * 非同期イベントハンドラの場合、関数はイベントハンドラで結果を通知します。
 * <ul>
 * <li>{function()} onNew -
 * インスタンスが使用可能になった時に呼び出されます。
 * </li>
 * <li>{function(v)} onRead  -
 * read関数のコールバック関数です。
 * 	<ul>
 * 		<li>{int} v - 現在のピンの値です。</li>
 * 	</ul>
 * </li>
 * <li>{function()} onMode -
 * mode関数のコールバック関数です。
 * </li>
 * </ul>
 * <p>
 * Generatorを指定した場合、コールバック関数の引数はyiledの戻り値として取得できます。
 * <p>
 * @return {mbedJS.BusIn}
 * @example //Callback
 * var mcu=new mbedJS.Mcu("192.168.128.39",
 * {
 *   onNew:function(){
 *     var pin=new mbedJS.BusIn(mcu,[mbedJS.PinName.P0_21,mbedJS.PinName.P0_22],{
 *     onNew:function(){
 *       pin.read();
 *     },
 *     onRead:function(v)
 *     {
 *       pin.mode(mbedJS.PinMode.PullDown);
 *     },
 *     onMode:function(v){
 *       mcu.close();
 *     }});
 *   },
 *   onClose:function(){
 *   },
 *   onError:function(){
 *   }
 * });
 * @example //Generator
 * var g=function*(){
 * try{
 *   var mcu=new mbedJS.Mcu("192.168.128.39",g);
 *   yield mcu.waitForNew();
 *   var bus=new mbedJS.BusIn(mcu,[mbedJS.PinName.P0_21,mbedJS.PinName.P0_22],g);
 *   yield bus.waitForNew();
 *   var v=yield bus.read();
 *   v=yield bus.mode(mbedJS.PinMode.PullUp);
 *   yield mcu.close();
 * }catch(e){
 *   mcu.shutdown();
 * }}();
 * g.next();
 */
var CLASS=function BusIn(i_mcu,i_params,i_handler)
{
	try{
		var _t=this;
		_t._mcu=i_mcu;
		_t._lc=CLASS;
		if(MI.isGenerator(i_handler)){_t._gen=i_handler;}
		else if(i_handler){_t._event=i_handler}
		function cb(j)
		{
			_t._oid=j.result[0];
			if(_t._event.onNew){_t._event.onNew();}
			if(_t._gen){_t._gen.next(_t);}
			_t._lc=null;
		}
		//Pin配列の正規化
		var ap=i_params;
		if(ap.length<1 ||ap.length>16){
			throw new MI.MiMicException(MI.NG_INVALID_ARG);
		}
		//数値のみの配列かな？
		MI.assertInt(ap);
		var pins=ap[0];
		var i=1;
		for(;i<i_params.length;i++){
			pins+=","+ap[i];
		}
		for(;i<16;i++){
			pins+=","+NS.PinName.NC;
		}
		return _t._mcu.rpc(_t.RPC_NS+":_new1",pins,cb);
	}catch(e){
		throw new MI.MiMicException(e);
	}
}
CLASS.prototype=
{
	/** @private */
	RPC_NS:"mbedJS:BusIn",
	/** @private 最後にコールしたAPIです。*/
	_lc:null,
	/** @private Generatorモードの場合にGeneratorオブジェクトを保持します。*/
	_gen:null,
	/** @private コールバック関数の連想配列です。要素はコンストラクタを参照してください。*/
	_event:{},
	/** @private リモートインスタンスのオブジェクトIDです。*/
	_oid:null,
	/**
	 * Generatorモードのときに使用する関数です。
	 * Generatorモードの時は、yieldと併用してnew BusIn()の完了を待ちます。
	 * @name mbedJS.BusIn#waitForNew
	 * @function
	 */
	waitForNew:function BusIn_waitForNew()
	{
		try{
			if(this._lc!=CLASS){throw new MI.MiMicException(MI.Error.NG_ILLEGAL_CALL);}
			this._lc=CLASS.waitForNew;
		}catch(e){
			throw new MI.MiMicException(e);
		}
	},
	/**
	 * ピンから値を読み込みます。
	 * 関数の完了時にonReadイベントが発生します。
	 * Generatorモードの時は、yieldと併用して完了を待機できます。
	 * @name mbedJS.BusIn#read
	 * @function
	 * @return　{int}
	 * Callbackモードの時はRPCメソッドのインデクスを返します。
	 * @return　{int}
	 * Generatorモードの時はピンの値を返します。
	 */
	read:function BusIn_read()
	{
		try{
			var _t=this;
			if(this._gen && this._lc){throw new MI.MiMicException(MI.Error.NG_YIELD_NOT_COMPLETED);}
			_t._lc=CLASS.read;
			return _t._mcu.rpc(_t.RPC_NS+":read",_t._oid,
			function (j)
			{
				var v=j.result[0];
				if(_t._event.onRead){_t._event.onRead(v);}
				if(_t._gen){_t._gen.next(v);}
				 _t._lc=null;
			}
			);
		}catch(e){
			throw new MI.MiMicException(e);
		}
	},
	/**
	 * ピンモードを設定します。
	 * 関数の完了時にonModeイベントが発生します。
	 * Generatorモードの時は、yieldと併用して完了を待機できます。
	 * @name mbedJS.BusIn#mode
	 * @function
	 * @param {PinMode} i_value
	 * PinModeの値です。
	 * @return　{int}
	 * Callbackモードの時はRPCメソッドのインデクスを返します。
	 */
	mode:function BusIn_mode(i_value)
	{
		try{
			var _t=this;
			if(this._gen && this._lc){throw new MI.MiMicException(MI.Error.NG_YIELD_NOT_COMPLETED);}
			_t._lc=CLASS.mode;
			MI.assertInt(i_value);
			return _t._mcu.rpc(_t.RPC_NS+":mode",_t._oid+","+i_value,
			function (j)
			{
				var v=j.result[0];
				if(_t._event.onMode){_t._event.onMode(v);}
				if(_t._gen){_t._gen.next(v);}
				 _t._lc=null;
			}
			);
		}catch(e){
			throw new MI.MiMicException(e);
		}			
	}	
}
NS.BusIn=CLASS;
}());