/**
 * @fileOverview Memoryクラスを定義します。
 */
(function(){
var NS=mbedJS;
var MI=MiMicJS;

/**
 * Memoryクラスです。
 * <a href="https://mbed.org/handbook/Memory">mbed::Memory</a>と同等の機能を持ちます。
 * @name mbedJS.Memory
 * @constructor
 * @param {mbedJS.Mcu} i_mcu
 * インスタンスをバインドするMCUオブジェクトです。
 * @param {HashMap|Generator} i_event
 * 非同期イベントハンドラの連想配列、又はGeneratorです。
 * <p>
 * 非同期イベントハンドラの場合、関数はイベントハンドラで結果を通知します。
 * <ul>
 * <li>{function()} onNew -
 * インスタンスが使用可能になった時に呼び出されます。
 * </li>
 * <li>{function(v)} onRead  -
 * read関数のコールバック関数です。
 * 	<ul>
 * 		<li>{int} v - 現在のピンの値です。</li>
 * 	</ul>
 * </li>
 * <li>{function()} onMode -
 * mode関数のコールバック関数です。
 * </li>
 * </ul>
 * <p>
 * Generatorを指定した場合、コールバック関数の引数はyiledの戻り値として取得できます。
 * <p>
 * @return {mbedJS.Memory}
 * @example //Callback
 * @example //Generator
 */
var CLASS=function Memory(i_mcu,i_handler)
{
	try{
		var _t=this;
		_t._mcu=i_mcu;
		_t._lc=CLASS;
		if(MI.isGenerator(i_handler)){_t._gen=i_handler;}
		else if(i_handler){_t._event=i_handler}
		function cb(j)
		{
			if(_t._event.onNew){_t._event.onNew();}
			if(_t._gen){_t._gen.next(_t);}
			_t._lc=null;
		}
		return _t._mcu.rpc(_t.RPC_NS+":init","",cb);
	}catch(e){
		throw new MI.MiMicException(e);
	}
}
CLASS.prototype=
{
	/** @private */
	RPC_NS:"MiMic:Memory",
	/** @private 最後にコールしたAPIです。*/
	_lc:null,
	/** @private Generatorモードの場合にGeneratorオブジェクトを保持します。*/
	_gen:null,
	/** @private コールバック関数の連想配列です。要素はコンストラクタを参照してください。*/
	_event:{},
	/**
	 * Generatorモードのときに使用する関数です。
	 * Generatorモードの時は、yieldと併用してnew Memory()の完了を待ちます。
	 * @name mbedJS.Memory#waitForNew
	 * @function
	 */
	waitForNew:function Memory_waitForNew()
	{
		try{
			if(this._lc!=CLASS){throw new MI.MiMicException(MI.Error.NG_ILLEGAL_CALL);}
			this._lc=CLASS.waitForNew;
		}catch(e){
			throw new MI.MiMicException(e);
		}
	},
	/**
	 * 1バイト単位でメモリから読み込みます。
	 * 関数の完了時にonReadイベントが発生します。
	 * Generatorモードの時は、yieldと併用して完了を待機できます。
	 * @name mbedJS.Memory#read
	 * @function
	 * @param {int} i_addr
	 * メモリアドレス
	 * @param {int} i_size
	 * (Optional) 読出しサイズです。省略時は1です。
	 * @return　{int}
	 * Callbackモードの時はRPCメソッドのインデクスを返します。
	 * @return　{int[]}
	 * Generatorモードの時はメモリ値を格納した配列を返します。
	 */
	read:function Memory_read(i_addr,i_size)
	{
		//read(i_addr)
		//read(i_addr,i_len)
		try{
			var _t=this;
			if(this._gen && this._lc){throw new MI.MiMicException(MI.Error.NG_YIELD_NOT_COMPLETED);}
			_t._lc=CLASS.read;
			var a=[i_addr,arguments.length<2?1:i_size];
			MI.assertInt(a);
			return _t._mcu.rpc(_t.RPC_NS+":read",a[0]+","+a[1],
			function (j)
			{
				var v=MI.bstr2byteArray(j.result[0]);
				if(_t._event.onRead){_t._event.onRead(v);}
				if(_t._gen){_t._gen.next(v);}
				 _t._lc=null;
			});
		}catch(e){
			throw new MI.MiMicException(e);
		}
	},
	/**
	 * 1バイトをメモリへ書き込みます。
	 * 関数の完了時にonWriteイベントが発生します。
	 * Generatorモードの時は、yieldと併用して完了を待機できます。
	 * @name mbedJS.Memory#write
	 * @function
	 * @param {int} i_addr
	 * 書き込み先のメモリアドレスを指定します。
	 * @param {int|int[]} i_v
	 * 書き込むbyte配列、または数値を指定します。
	 * 数値の場合は1バイトを書き込みます。最大長さは200byteくらいです。
	 * @return　{int}
	 * Callbackモードの時はRPCメソッドのインデクスを返します。
	 */
	write:function Memory_write(i_addr,i_v)
	{
		try{
			var _t=this;
			if(this._gen && this._lc){throw new MI.MiMicException(MI.Error.NG_YIELD_NOT_COMPLETED);}
			_t._lc=CLASS.write;
			MI.assertInt(i_addr);
			MI.assertInt(i_v);
			return _t._mcu.rpc(_t.RPC_NS+":write",i_addr+",\""+MI.byteArray2bstr(i_v)+"\"",
			function (j)
			{
				if(_t._event.onWrite){_t._event.onWrite();}
				if(_t._gen){_t._gen.next();}
				 _t._lc=null;
			}
			);
		}catch(e){
			throw new MI.MiMicException(e);
		}
	},
	/**
	 * 4バイト単位でメモリから読み込みます。
	 * 関数の完了時にonRead32イベントが発生します。
	 * Generatorモードの時は、yieldと併用して完了を待機できます。
	 * @name mbedJS.Memory#read32
	 * @function
	 * @param {int} i_addr
	 * メモリアドレス
	 * @param {int} i_size
	 * (Optional) 読出しサイズです。省略時は4です。4の倍数を指定してください。
	 * @return　{int}
	 * Callbackモードの時はRPCメソッドのインデクスを返します。
	 * @return　{int[]}
	 * Generatorモードの時はメモリ値を格納した配列を返します。
	 */
	read32:function Memory_read32(i_addr,i_size)
	{
		//read(i_addr)
		//read(i_addr,i_len)
		try{
			var _t=this;
			if(this._gen && this._lc){throw new MI.MiMicException(MI.Error.NG_YIELD_NOT_COMPLETED);}
			_t._lc=CLASS.read32;
			var a=[i_addr,arguments.length<2?4:i_size];
			if(a[1]%4!=0){
				throw new MI.MiMicException(MI.Error.NG_INVALID_ARG);
			}
			MI.assertInt(a);
			return _t._mcu.rpc(_t.RPC_NS+":read",a[0]+","+a[1],
			function (j)
			{
				var v=MI.bstr2uintArray(j.result[0]);
				if(_t._event.onRead32){_t._event.onRead32(v);}
				if(_t._gen){_t._gen.next(v);}
				 _t._lc=null;
			});
		}catch(e){
			throw new MI.MiMicException(e);
		}
	},
	/**
	 * 32bit unsigned intをメモリへ書き込みます。
	 * 関数の完了時にonWrite32イベントが発生します。
	 * Generatorモードの時は、yieldと併用して完了を待機できます。
	 * @name mbedJS.Memory#write32
	 * @function
	 * @param {int} i_addr
	 * 書き込み先のメモリアドレスを指定します。
	 * @param {int|int[]} i_v
	 * 書き込むbyte配列、または数値を指定します。
	 * 数値の場合は1バイトを書き込みます。最大長さは200byteくらいです。
	 * @return　{int}
	 * Callbackモードの時はRPCメソッドのインデクスを返します。
	 */
	write32:function Memory_write32(i_addr,i_v)
	{
		try{
			var _t=this;
			if(this._gen && this._lc){throw new MI.MiMicException(MI.Error.NG_YIELD_NOT_COMPLETED);}
			_t._lc=CLASS.write32;
			MI.assertInt(i_addr);
			MI.assertInt(i_v);
			return _t._mcu.rpc(_t.RPC_NS+":write",i_addr+",\""+MI.uintArray2bstr(i_v)+"\"",
			function (j)
			{
				if(_t._event.onWrite){_t._event.onWrite32();}
				if(_t._gen){_t._gen.next();}
				 _t._lc=null;
			}
			);
		}catch(e){
			throw new MI.MiMicException(e);
		}
	}
}
NS.Memory=CLASS;
}());